/*
Copyright (c) 2018 Vereign AG [https://www.vereign.com]

This is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

package main

import (
	"flag"

	"code.vereign.com/code/viam-apis/log"

	"code.vereign.com/code/key-storage-agent/config"
	"code.vereign.com/code/key-storage-agent/server"
)

// main start a gRPC server and waits for connection
func main() {
	//log.SetFlags(log.LstdFlags | log.Lshortfile)

	configFile := flag.String("config-file", "", "path to configuration file")
	etcdURL := flag.String("etcd-url", "", "etcd URL")
	flag.Parse()

	if *configFile == "" && *etcdURL == "" {
		log.Fatal("Config file path or etcd URL not specified")
		return
	}

	config.SetConfigValues(*configFile, *etcdURL)

	grpcAddress := config.GrpcListenAddress
	restAddress := config.RestListenAddress
	dataStorageAddress := config.DataStorageUrl
	certPem := config.CertificatePEM
	keyPem := config.PrivateKeyPEM
	caCertPem := config.CaCertificatePEM
	vereignCaCertificatePem := config.VereignCaCertificatePEM
	vereignCaKeyPem := config.VereignCaKeyPEM

	maxMessageSize := config.MaxMessageSize
	log.SetConfiguration(config.GlobalLogLevel)

	// fire the gRPC server in a goroutine
	go func() {
		err := server.StartGRPCServer(grpcAddress, certPem, keyPem, caCertPem, vereignCaCertificatePem,
			vereignCaKeyPem, dataStorageAddress, maxMessageSize)
		if err != nil {
			log.Fatalf("failed to start gRPC server: %s", err)
		}
	}()

	// fire the REST server in a goroutine
	go func() {
		err := server.StartRESTServer(restAddress, grpcAddress, certPem)
		if err != nil {
			log.Fatalf("failed to start gRPC server: %s", err)
		}
	}()

	// infinite loop
	log.Printf("Entering infinite loop")
	select {}
}
